"use strict";

var Promise = require("bluebird");

require("../mocha-config.js");

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AliasSurface = require('../../../main/js/map/surface/AliasSurface');
var Comment = require('../../../main/js/map/data/Comment');
var CustomMap = require('../../../main/js/map/CustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');
var MapContextMenu = require('../../../main/js/gui/MapContextMenu');
var PointData = require('../../../main/js/map/data/PointData');
var PointMarker = require('../../../main/js/map/marker/PointMarker');
var ReactionMarker = require('../../../main/js/map/marker/ReactionMarker');
var ReactionSurface = require('../../../main/js/map/surface/ReactionSurface');
var SelectionContextMenu = require('../../../main/js/gui/SelectionContextMenu');

var ServerConnector = require('./../ServerConnector-mock');

var logger = require('./../logger');

var chai = require('chai');
var assert = chai.assert;

describe('CustomMap', function () {
  describe("constructor", function () {
    it("default", function () {
      var options = helper.createCustomMapOptions();
      var map = new CustomMap(options);
      assert.ok(map);
    });

    it("with submaps", function () {
      var options = helper.createCustomMapOptions();

      options.getProject().getModel().addSubmodel(helper.createModel());

      var map = new CustomMap(options);
      assert.ok(map);
    });

    it("with session data pointing to not existing overlay", function () {
      var options = helper.createCustomMapOptions();

      ServerConnector.getSessionData(options.getProject()).setSelectedBackgroundOverlay("-1");

      var map = new CustomMap(options);
      assert.ok(map);
    });
  });

  it("getSubmapById", function () {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmapById(map.getId()));
  });

  it("getSubmapById (invalid)", function () {
    var map = helper.createCustomMap();
    assert.equal(map.getSubmapById(-1), null);
  });

  it("getSubmapById (string id)", function () {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmapById(map.getId() + ""));
  });

  describe("openDataOverlay", function () {
    it("for not existing id", function () {
      var map = helper.createCustomMap();
      try {
        map.openDataOverlay(-1);
        assert.ok(false);
      } catch (exception) {
        assert.ok(exception.message.indexOf("You have no privileges") >= 0);
      }
    });

    it("for int id", function () {
      var options = helper.createCustomMapOptions();
      var layout = options.getProject().getDataOverlays()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout.getId());
      assert.equal(logger.getErrors().length, 0);
    });

    it("for overlay object", function () {
      var options = helper.createCustomMapOptions();
      var layout = options.getProject().getDataOverlays()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout);
      assert.equal(logger.getErrors().length, 0);
    });

    it("for string id", function () {
      var options = helper.createCustomMapOptions();
      var layout = options.getProject().getDataOverlays()[0];
      var map = new CustomMap(options);
      map.openDataOverlay(layout.getId() + "");
      assert.equal(logger.getErrors().length, 0);
    });

    it("with background overlay", function () {
      var map = helper.createCustomMap();
      var layout = helper.createOverlay();
      layout.setInitialized(true);
      layout.setInputDataAvailable(false);
      map.getProject().addDataOverlay(layout);

      return map.openDataOverlay(layout.getId()).then(function () {
        var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
        assert.equal(0, vLayouts.length);
      });
    });

    it("with non background overlay", function () {
      var map = helper.createCustomMap();
      var layout = helper.createOverlay();
      layout.setInitialized(true);
      layout.setInputDataAvailable(true);
      map.getProject().addDataOverlay(layout);

      var emptyBackground = helper.createOverlay();
      emptyBackground.setInputDataAvailable(false);
      emptyBackground.setName("Empty");
      map.getProject().addDataOverlay(emptyBackground);

      return map.openDataOverlay(layout.getId()).then(function () {
        var backgroundId = ServerConnector.getSessionData(map.getProject()).getSelectedBackgroundOverlay();
        assert.equal(backgroundId, emptyBackground.getId());
      });
    });

    it("simple", function () {
      var map = helper.createCustomMap();
      var alias = helper.createAlias(map);
      map.getModel().addAlias(alias);

      var reaction = helper.createReaction(map);
      map.getModel().addReaction(reaction);

      var layout = helper.createOverlay();
      layout.setInputDataAvailable(true);
      layout.setInitialized(true);
      var layoutAlias = helper.createLayoutAlias(alias);
      layout.addAlias(layoutAlias);

      var layoutReaction = helper.createLayoutReaction(reaction);
      layout.addReaction(layoutReaction);

      map.getProject().addDataOverlay(layout);

      return map.openDataOverlay(layout.getId()).then(function () {
        return map._showSelectedDataOverlay(layout.getId(), 0, 1);
      }).then(function () {
        var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
        assert.equal(1, vLayouts.length);
        assert.equal(layout.getId(), vLayouts[0]);

        assert.equal(2, map.selectedLayoutOverlays[layout.getId()].length);
        assert.ok(map.selectedLayoutOverlays[layout.getId()][0] instanceof AliasSurface);
        assert.ok(map.selectedLayoutOverlays[layout.getId()][1] instanceof ReactionSurface);

        // now hide the layout
        map._hideSelectedLayout(layout.getId());
        assert.ok(map.selectedLayoutOverlays[layout.getId()]);
        assert.equal(0, map.selectedLayoutOverlays[layout.getId()].length);
      });

    });

    it("with submaps", function () {
      var projectId = "complex_model_with_submaps";
      helper.setUrl("http://test/?id=" + projectId);
      var customMap;
      var emptySubmodelId = 16730;
      var filledSubmodelId = 16731;
      var overlayId = 18083;

      return ServerConnector.getProject(projectId).then(function (project) {
        var options = helper.createCustomMapOptions(project);
        customMap = new CustomMap(options);

        return customMap.openSubmap(emptySubmodelId);
      }).then(function () {
        return customMap.openSubmap(filledSubmodelId);
      }).then(function () {

        var emptySubmap = customMap.getSubmapById(emptySubmodelId);
        var filledSubmap = customMap.getSubmapById(filledSubmodelId);

        return customMap.openDataOverlay(overlayId).then(function () {

          assert.equal(1, customMap.selectedLayoutOverlays[overlayId].length);
          assert.equal(1, filledSubmap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, emptySubmap.selectedLayoutOverlays[overlayId].length);

          // now hide the layout
          customMap.hideSelectedLayout(overlayId);
          assert.equal(0, customMap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, filledSubmap.selectedLayoutOverlays[overlayId].length);
          assert.equal(0, emptySubmap.selectedLayoutOverlays[overlayId].length);
        });

      }).then(function () {
        return customMap.destroy();
      });
    });

    it("with submap double opened", function () {
      var projectId = "complex_model_with_submaps";
      helper.setUrl("http://test/?id=" + projectId);
      var customMap, filledSubmap;
      var filledSubmodelId = 16731;
      var overlayId = 18083;

      return ServerConnector.getProject(projectId).then(function (project) {
        customMap = helper.createCustomMap(project);
        return customMap.openSubmap(filledSubmodelId);
      }).then(function () {
        filledSubmap = customMap.getSubmapById(filledSubmodelId);
        return customMap.openDataOverlay(overlayId);
      }).then(function () {
        assert.equal(1, filledSubmap.selectedLayoutOverlays[overlayId].length);
        return customMap.openSubmap(filledSubmodelId);
      }).then(function () {
        //after opening submap nothing should change
        assert.equal(1, filledSubmap.selectedLayoutOverlays[overlayId].length);
      }).finally(function () {
        return customMap.destroy();
      });
    });

  });

  it("hideDataOverlay", function () {
    var map = helper.createCustomMap();
    var layout = helper.createOverlay();
    layout.setInitialized(true);
    layout.setInputDataAvailable(true);
    map.getProject().addDataOverlay(layout);

    return map.openDataOverlay(layout.getId()).then(function () {
      return map.hideDataOverlay(layout.getId());
    }).then(function () {
      var vLayouts = ServerConnector.getSessionData(map.getProject()).getVisibleOverlays();
      assert.equal(0, vLayouts.length);
    });
  });

  describe("openInfoWindowForAlias", function () {
    it("default", function () {
      var map = helper.createCustomMap();
      var layout = helper.createOverlay();
      var alias = helper.createAlias();
      alias.setModelId(map.getId());
      var layoutAlias = helper.createLayoutAlias();
      layoutAlias.setId(alias.getId());

      layout.addAlias(layoutAlias);

      // create layout
      map.getProject().addDataOverlay(layout);
      map.getModel().addAlias(alias);

      return map.openInfoWindowForAlias(alias).then(function () {
        assert.ok(map.getAliasInfoWindowById(alias.getId()));
        assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
      });
    });

    it("for incomplete alias", function () {
      var map, alias;
      return ServerConnector.getProject().then(function (project) {
        var options = helper.createCustomMapOptions(project);
        map = new CustomMap(options);

        return map.getModel().getAliasById(329171, false);
      }).then(function (result) {
        alias = result;
        assert.notOk(alias.isComplete());
        return map.openInfoWindowForAlias(alias);
      }).then(function () {
        assert.ok(map.getAliasInfoWindowById(alias.getId()));
        assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
      });
    });
  });


  describe("renderOverlayCollection", function () {
    it("for alias", function () {
      var map = helper.createCustomMap();
      var alias = helper.createAlias(map);

      var oc = helper.createDbOverlay(map);

      oc.getIdentifiedElements = function () {
        return Promise.resolve([new IdentifiedElement({
          objectId: alias.getId(),
          icon: "empty.png",
          modelId: map.getId(),
          type: "Alias"
        })]);
      };
      return map.renderOverlayCollection({
        overlayCollection: oc
      }).then(function () {
        var markers = map.getMarkerSurfaceCollection().getMarkers();
        assert.equal(1, markers.length);
      });
    });

    it("alias re-rendering with different icon", function () {
      var map = helper.createCustomMap();
      var reaction = helper.createReaction(map, true);
      var alias = helper.createAlias(map);
      map.getModel().addAlias(alias);
      map.getModel().addReaction(reaction);

      var identifiedElement = new IdentifiedElement(alias);
      identifiedElement.setIcon("empty.png");
      var identifiedElement2 = new IdentifiedElement(alias);
      identifiedElement2.setIcon("new_icon.png");

      var marker;
      var oc = helper.createDbOverlay(map);

      oc.getIdentifiedElements = function () {
        return Promise.resolve([identifiedElement, new IdentifiedElement(reaction)]);
      };
      return map.renderOverlayCollection({
        overlayCollection: oc
      }).then(function () {
        marker = map.getMarkerSurfaceCollection().getMarker(identifiedElement);
        oc.getIdentifiedElements = function () {
          return Promise.resolve([identifiedElement2]);
        };
        assert.ok(map.getMarkerSurfaceCollection().getSurface({element: reaction, overlay: oc}));

        return map.renderOverlayCollection({
          overlayCollection: oc
        });
      }).then(function () {
        assert.equal(marker.getIcon(), "new_icon.png");

        assert.notOk(oc.mapOverlays["REACTION"][reaction.getId()]);
      });
    });

    it("for set of aliases", function () {
      var map = helper.createCustomMap();

      var oc = helper.createDbOverlay(map);

      var identifiedElements = [];
      for (var i = 0; i < 3; i++) {
        var alias = helper.createAlias(map);
        map.getModel().addAlias(alias);
        var ie = helper.createIdentifiedElement(alias);
        ie.setIcon("");
        identifiedElements.push(ie);
      }

      oc.getIdentifiedElements = function () {
        return Promise.resolve(identifiedElements);
      };

      return map.renderOverlayCollection({
        overlayCollection: oc
      }).then(function () {
        var markers = map.getMarkerSurfaceCollection().getMarkers();
        assert.equal(3, markers.length);
      });
    });

    it("for point", function () {
      var map = helper.createCustomMap();

      var oc = helper.createDbOverlay(map);

      var javaObj = {
        objectId: "Point2D.Double[117.685546875, 204.6923828125001]",
        modelId: map.getId(),
        type: "POINT",
        icon: "marker/empty.png"
      };

      oc.getIdentifiedElements = function () {
        return Promise.resolve([new IdentifiedElement(javaObj)]);
      };

      return map.renderOverlayCollection({
        overlayCollection: oc
      }).then(function () {
        var markers = map.getMarkerSurfaceCollection().getMarkers();
        assert.equal(1, markers.length);
      });
    });

    it("for reaction", function () {
      var map = helper.createCustomMap();
      var reaction = helper.createReaction(map, true);
      map.getModel().addReaction(reaction);

      var oc = helper.createDbOverlay(map);

      oc.getIdentifiedElements = function () {
        return Promise.resolve([new IdentifiedElement(reaction)]);
      };

      return map.renderOverlayCollection({
        overlayCollection: oc
      }).then(function () {
        var surfaces = map.getMarkerSurfaceCollection().getSurfaces();
        assert.equal(1, surfaces.length);
      });
    });
  });

  it("refreshMarkers", function () {
    var map = helper.createCustomMap();
    var alias = helper.createAlias(map);
    map.getModel().addAlias(alias);

    var oc = helper.createDbOverlay(map);

    oc.getIdentifiedElements = function () {
      var element = new IdentifiedElement(alias);
      element.setIcon("icon");
      return Promise.resolve([element]);
    };

    return map.renderOverlayCollection({
      overlayCollection: oc
    }).then(function () {
      return map.refreshMarkers(true);
    }).then(function () {
      var markers = map.getMarkerSurfaceCollection().getMarkers();
      assert.equal(1, markers.length);
    });
  });

  it("clearDbOverlays", function () {
    var map = helper.createCustomMap();

    var oc = helper.createDbOverlay(map);

    var javaObj = {
      objectId: "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId: map.getId(),
      type: "POINT",
      icon: "marker/empty.png"
    };
    var searchResults = [new IdentifiedElement(javaObj)];

    oc.getIdentifiedElements = function () {
      return Promise.resolve(searchResults);
    };
    oc.clear = function () {
      searchResults = [];
      return this.callListeners("onSearch", searchResults);
    };

    return map.renderOverlayCollection({
      overlayCollection: oc
    }).then(function () {

      return map.clearDbOverlays();
    }).then(function () {
      var markerCount = 0;
      for (var id in oc.pointMarkers) {
        if (oc.pointMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }

      assert.equal(0, markerCount);
    });

  });

  it("getInfoWindowForIdentifiedElement ( reaction)", function () {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction(map);
    map.getModel().addReaction(reaction);

    var ie = helper.createIdentifiedElement(reaction);

    var infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.equal(null, infoWindow);
    return map._openInfoWindowForReaction(reaction).then(function () {
      infoWindow = map.getInfoWindowForIdentifiedElement(ie);

      assert.ok(infoWindow);
    });

  });

  it("right click on map", function () {
    var map = helper.createCustomMap();
    map.setContextMenu(new MapContextMenu({
      customMap: map,
      element: testDiv
    }));

    var mev = {
      stop: null,
      latLng: new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'rightclick', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });

  it("left click on map", function () {
    var map;
    var searchOverlay;
    return ServerConnector.getProject().then(function (project) {
      map = helper.createCustomMap(project);
      searchOverlay = helper.createSearchDbOverlay(map);

      var mev = {
        stop: null,
        latLng: new google.maps.LatLng(82.32061703407554, -167.25586206896548)
      };
      map.getGoogleMap().setZoom(4);
      // latLng : new google.maps.LatLng(82.40238643645326, -148.44758620689652)

      assert.notOk(map.getActiveSubmapId());
      return google.maps.event.trigger(map.getGoogleMap(), 'click', mev);

    }).then(function () {
      assert.equal(map.getId(), map.getActiveSubmapId());
      var element = new IdentifiedElement({id: 329171, type: "ALIAS", modelId: map.getId()});
      assert.ok(map.getMarkerSurfaceCollection().getMarker(element));
    });

  });

  it("left click on reaction", function () {
    var map;
    var searchOverlay;
    return ServerConnector.getProject().then(function (project) {
      map = helper.createCustomMap(project);
      searchOverlay = helper.createSearchDbOverlay(map);

      var mev = {
        stop: null,
        latLng: new google.maps.LatLng(82.40238643645326, -148.44758620689652)
      };
      map.getGoogleMap().setZoom(4);

      assert.notOk(map.getActiveSubmapId());
      return google.maps.event.trigger(map.getGoogleMap(), 'click', mev);
    }).then(function () {
      assert.equal(map.getId(), map.getActiveSubmapId());

      var reaction = new IdentifiedElement({id: 153521, type: "REACTION", modelId: map.getId()});
      var surface = map.getMarkerSurfaceCollection().getSurface({element: reaction, overlay: searchOverlay});

      assert.ok(surface);
      assert.ok(surface.isShown());

      var element = new IdentifiedElement({id: 329165, type: "ALIAS", modelId: map.getId()});
      var marker = map.getMarkerSurfaceCollection().getMarker(element);
      assert.ok(marker);
    });
  });

  it("create polygon through drawingManager", function () {
    var map = helper.createCustomMap();

    map.setSelectionContextMenu(new SelectionContextMenu({
      customMap: map,
      element: testDiv
    }));

    map.turnOnDrawing();

    var triangleCoordinates = [new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
      new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190)];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths: triangleCoordinates,
      strokeColor: '#FF0000',
      strokeOpacity: 0.8,
      strokeWeight: 2,
      fillColor: '#FF0000',
      fillOpacity: 0.35
    });
    bermudaTriangle.setMap(map.getGoogleMap());

    var eventParam = {
      type: google.maps.drawing.OverlayType.POLYGON,
      overlay: bermudaTriangle
    };

    // check creation complete behaviour
    assert.ok(map.isDrawingOn());
    assert.equal(google.maps.drawing.OverlayType.POLYGON, map._drawingManager.getDrawingMode());
    google.maps.event.trigger(map._drawingManager, 'overlaycomplete', eventParam);
    assert.ok(map.isDrawingOn());
    assert.equal(null, map._drawingManager.getDrawingMode());

    // and now check how it behaves when we click on the polygon
    var clickEventParam = {
      stop: null,
      latLng: new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getSelectedPolygon());

    assert.notOk(map.getActiveSubmapId());
    assert.notOk(map.getSelectedArea());
    google.maps.event.trigger(bermudaTriangle, 'rightclick', clickEventParam);
    assert.ok(map.getSelectedArea());
    assert.equal(map.getId(), map.getActiveSubmapId());

    assert.ok(map.getSelectedPolygon());

  });

  it("getAliasVisibleLayoutsData", function () {
    var mockObject = helper.createCustomMap();

    var alias = helper.createAlias();
    mockObject.getModel().addAlias(alias);

    var layout = helper.createOverlay();
    layout.setInputDataAvailable(true);
    layout.setInitialized(true);
    mockObject.getProject().addDataOverlay(layout);

    return mockObject.openDataOverlay(layout.getId()).then(function () {
      return mockObject.getAliasVisibleLayoutsData(alias.getId());
    }).then(function (layoutAliases) {
      assert.equal(layoutAliases.length, 1);
    });
  });

  it("changed coordinates in map", function () {
    var map = helper.createCustomMap();
    var oldCenter = map.getGoogleMap().getCenter();
    var newCenter = new google.maps.LatLng(3, 87);
    map.getGoogleMap().setCenter(newCenter);
    google.maps.event.trigger(map.getGoogleMap(), "center_changed");

    var center = ServerConnector.getSessionData(map.getProject()).getCenter(map.getModel());
    assert.ok(center !== oldCenter);
    assert.ok(center instanceof google.maps.Point);
  });

  it("refreshComments", function () {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function () {
      assert.notOk(map.getMarkerSurfaceCollection().getMarker(new IdentifiedElement({
        id: '(241.01,372.35)',
        modelId: map.getId(),
        type: "POINT"
      })));
      assert.ok(map.getMarkerSurfaceCollection().getMarker(new IdentifiedElement({
        id: '(643.96,144.09)',
        modelId: map.getId(),
        type: "POINT"
      })));
      assert.notOk(map.getMarkerSurfaceCollection().getMarker(new IdentifiedElement({
        id: '(216.65,370.00)',
        modelId: map.getId(),
        type: "POINT"
      })));
      assert.notOk(map.getMarkerSurfaceCollection().getMarker(new IdentifiedElement({
        id: 'unkId',
        modelId: map.getId(),
        type: "POINT"
      })));
    });
  });

  it("hide comments", function () {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    helper.createCommentDbOverlay(map);

    ServerConnector.getSessionData().setShowComments(true);
    return map.refreshComments().then(function () {
      ServerConnector.getSessionData().setShowComments(false);
      return map.refreshComments();
    }).then(function () {
      assert.notOk(map.getMarkerSurfaceCollection().getMarker(new IdentifiedElement({
        id: '(241.01, 372.35)',
        modelId: map.getId(),
        type: "POINT"
      })));
    });
  });

  it("openCommentDialog", function () {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    map.setActiveSubmapId(15781);
    map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
    return map.openCommentDialog().then(function () {
      var types = map.getCommentDialog().getTypes();
      assert.equal(types.length, 6);
      var selected = map.getCommentDialog().getSelectedType();
      assert.ok(selected === "<General>");

      map.getCommentDialog().setSelectedType(1);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");

      map.getCommentDialog().setSelectedType(2);
      selected = map.getCommentDialog().getSelectedType();
      assert.notOk(selected === "<General>");
    }).then(function () {
      map.getCommentDialog().destroy();
    });
  });

  it("addComment", function () {
    var map;
    return ServerConnector.getProject().then(function (project) {
      map = helper.createCustomMap(project);
      helper.createCommentDbOverlay(map);
      map.setActiveSubmapId(map.getId());
      map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
      return map.openCommentDialog();
    }).then(function () {
      return map.getCommentDialog().addComment();
    }).then(function () {
      map.getCommentDialog().destroy();
    });
  });

  it("addComment for protein", function () {
    var map;
    return ServerConnector.getProject().then(function (project) {
      map = helper.createCustomMap(project);
      helper.createCommentDbOverlay(map);
      map.setActiveSubmapId(map.getId());
      map.setActiveSubmapClickCoordinates(new google.maps.Point(2, 12));
      return map.openCommentDialog();
    }).then(function () {
      var dialog = map.getCommentDialog();
      dialog.setSelectedType(1);
      return dialog.addComment();
    }).then(function () {
      map.getCommentDialog().destroy();
    });
  });

  it("retrieveOverlayDetailDataForElement for comment", function () {
    var map = helper.createCustomMap();
    helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329157);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.retrieveOverlayDetailDataForElement(ie, {
      comment: true
    }).then(function (details) {
      assert.ok(details);
      assert.equal(details.length, 1);
      assert.equal(details[0].length, 1);
      assert.ok(details[0][0] instanceof Comment);
    });
  });

  it("getOverlayDataForIdentifiedElement", function () {
    var map = helper.createCustomMap();
    var commentOverlay = helper.createCommentDbOverlay(map);

    var alias = helper.createAlias(map);
    alias.setId(329157);
    map.getModel().addAlias(alias);

    var ie = new IdentifiedElement(alias);

    return map.getOverlayDataForIdentifiedElement(ie, {
      comment: true
    }).then(function (details) {
      assert.equal(details.length, 1);

      var overlayData = details[0];
      assert.equal(overlayData.overlay, commentOverlay);
      assert.ok(overlayData.data);
      assert.equal(overlayData.data.length, 1);
      assert.ok(overlayData.data[0] instanceof Comment);
    });
  });

  it("openSubmap", function () {
    var options = helper.createCustomMapOptions();

    var submodel = helper.createModel();
    options.getProject().getModel().addSubmodel(submodel);

    var map = new CustomMap(options);

    map.openSubmap(submodel.getId());
    map.destroy();
  });

  describe("setCenter", function () {
    it("default", function () {
      var options = helper.createCustomMapOptions();

      var map = new CustomMap(options);

      map.setCenter(new google.maps.LatLng(10, 20));
      assert.ok(ServerConnector.getSessionData().getCenter(map.getModel()));
    });

    it("on submap", function () {
      var options = helper.createCustomMapOptions();

      var submodel = helper.createModel();
      options.getProject().getModel().addSubmodel(submodel);

      var map = new CustomMap(options);
      map.openSubmap(submodel.getId());
      map.setCenter(new google.maps.LatLng(10, 20));
      assert.ok(ServerConnector.getSessionData().getCenter(submodel));
      map.destroy();
    });
  });

  it("setZoom", function () {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);

    map.setZoom(3);
    assert.equal(3, ServerConnector.getSessionData().getZoomLevel(map.getModel()));
  });

  it("removeSelection when invalid model selected", function () {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);
    map.setActiveSubmapId(-1);

    try {
      map.removeSelection();
      assert.ok(false);
    } catch (e) {
      assert.ok(e.message.indexOf("Cannot find submap") >= 0);
    }
  });

  it("removeSelection", function () {
    var options = helper.createCustomMapOptions();

    var map = new CustomMap(options);
    map.setActiveSubmapId(map.getId());
    map.setSelectedArea(new google.maps.Rectangle({}));

    map.removeSelection();
    assert.equal(map.getSelectedArea(), null);
  });


  describe("_openInfoWindowForIdentifiedElement", function () {
    it("for AliasMarker", function () {
      var map;
      var alias, marker;
      return ServerConnector.getProject().then(function (project) {
        map = helper.createCustomMap(project);
        return map.getModel().getAliasById(329171);
      }).then(function (result) {
        alias = result;

        var identifiedElement = new IdentifiedElement(alias);
        identifiedElement.setIcon("empty.png");

        marker = new AliasMarker({
          element: identifiedElement,
          map: map
        });

        return marker.init();
      }).then(function () {
        assert.equal(null, map.getAliasInfoWindowById(alias.getId()));
        return map._openInfoWindowForIdentifiedElement(marker).then(function () {
          assert.ok(map.getAliasInfoWindowById(alias.getId()));
        });
      });

    });
    it("for ReactionMarker", function () {
      var map;
      var reaction, marker;
      return ServerConnector.getProject().then(function (project) {
        map = helper.createCustomMap(project);
        return map.getModel().getReactionById(153510);
      }).then(function (result) {
        reaction = result;

        marker = new ReactionMarker({
          element: new IdentifiedElement(reaction),
          map: map
        });
        return marker.init();
      }).then(function () {
        assert.equal(null, map.getReactionInfoWindowById(reaction.getId()));
        return map._openInfoWindowForIdentifiedElement(marker);
      }).then(function () {
        assert.ok(map.getReactionInfoWindowById(reaction.getId()));
      });

    });
    it("for PointMarker", function () {

      var mockObject = helper.createCustomMap();

      mockObject.getOverlayDataForPoint = function () {
        return Promise.resolve([]);
      };

      var point = new google.maps.Point(2, 3.45);
      var pointData = new PointData(point, mockObject.getId());

      var pointMarker = new PointMarker({
        element: new IdentifiedElement(pointData),
        map: mockObject
      });

      return pointMarker.init().then(function () {
        assert.equal(null, mockObject.getPointInfoWindowById(pointData.getId()));
        return mockObject._openInfoWindowForIdentifiedElement(pointMarker.getIdentifiedElement());
      }).then(function () {
        assert.ok(mockObject.getPointInfoWindowById(pointData.getId()));
      });
    });
  });

  describe("init", function () {
    it("invalid background overlay", function () {
      var options = helper.createCustomMapOptions();
      ServerConnector.getSessionData(options.getProject()).setSelectedBackgroundOverlay("bla");
      var map = new CustomMap(options);
      return map.init().then(function () {
        assert.false("Error expected");
      }, function () {
        map.destroy();
      })
    });

    it("non-existing background overlay", function () {
      var options = helper.createCustomMapOptions();
      ServerConnector.getSessionData(options.getProject()).setSelectedBackgroundOverlay(-1);
      var map = new CustomMap(options);
      return map.init().then(function () {
        assert.ok(ServerConnector.getSessionData(options.getProject()).getSelectedBackgroundOverlay() > 0);
        return map.destroy();
      })
    });

    it("with session data overlays", function () {
      var project = helper.createProject();
      var overlay1 = helper.createOverlay();
      overlay1.setInputDataAvailable(true);
      project.addDataOverlay(overlay1);

      var options = helper.createCustomMapOptions(project);
      ServerConnector.getSessionData(options.getProject()).setVisibleOverlays([overlay1.getId()]);
      var map = new CustomMap(options);
      return map.init().then(function () {
        return map.getVisibleDataOverlays();
      }).then(function (overlays) {
        assert.equal(1, overlays.length);
        map.destroy();
      })
    });

  });
});
